@@ -17,8 +17,8 @@ namespace icinga
 {
 
 static bool l_SSLInitialized = false;
-static boost::mutex *l_Mutexes;
-static boost::mutex l_RandomMutex;
+static std::mutex *l_Mutexes;
+static std::mutex l_RandomMutex;
 
 String GetOpenSSLVersion()
 {
@@ -62,7 +62,7 @@ void InitializeOpenSSL()
 	SSL_COMP_get_compression_methods();
 
 #ifdef CRYPTO_LOCK
-	l_Mutexes = new boost::mutex[CRYPTO_num_locks()];
+	l_Mutexes = new std::mutex[CRYPTO_num_locks()];
 	CRYPTO_set_locking_callback(&OpenSSLLockingCallback);
 	CRYPTO_set_id_callback(&OpenSSLIDCallback);
 #endif /* CRYPTO_LOCK */
@@ -70,7 +70,7 @@ void InitializeOpenSSL()
 	l_SSLInitialized = true;
 }
 
-static void SetupSslContext(const Shared<boost::asio::ssl::context>::Ptr& context, const String& pubkey, const String& privkey, const String& cakey)
+static void InitSslContext(const Shared<boost::asio::ssl::context>::Ptr& context, const String& pubkey, const String& privkey, const String& cakey)
 {
 	char errbuf[256];
 
@@ -137,7 +137,16 @@ static void SetupSslContext(const Shared<boost::asio::ssl::context>::Ptr& contex
 		}
 	}
 
-	if (!cakey.IsEmpty()) {
+	if (cakey.IsEmpty()) {
+		if (!SSL_CTX_set_default_verify_paths(sslContext)) {
+			ERR_error_string_n(ERR_peek_error(), errbuf, sizeof errbuf);
+			Log(LogCritical, "SSL")
+				<< "Error loading system's root CAs: " << ERR_peek_error() << ", \"" << errbuf << "\"";
+			BOOST_THROW_EXCEPTION(openssl_error()
+				<< boost::errinfo_api_function("SSL_CTX_set_default_verify_paths")
+				<< errinfo_openssl_error(ERR_peek_error()));
+		}
+	} else {
 		if (!SSL_CTX_load_verify_locations(sslContext, cakey.CStr(), nullptr)) {
 			ERR_error_string_n(ERR_peek_error(), errbuf, sizeof errbuf);
 			Log(LogCritical, "SSL")
@@ -179,9 +188,9 @@ Shared<boost::asio::ssl::context>::Ptr MakeAsioSslContext(const String& pubkey,
 
 	InitializeOpenSSL();
 
-	auto context (Shared<ssl::context>::Make(ssl::context::tlsv12));
+	auto context (Shared<ssl::context>::Make(ssl::context::tls));
 
-	SetupSslContext(context, pubkey, privkey, cakey);
+	InitSslContext(context, pubkey, privkey, cakey);
 
 	return context;
 }
@@ -227,6 +236,71 @@ void SetCipherListToSSLContext(const Shared<boost::asio::ssl::context>::Ptr& con
 #endif /* OPENSSL_VERSION_NUMBER >= 0x10100000L */
 }
 
+/**
+ * Resolves a string describing a TLS protocol version to the value of a TLS*_VERSION macro of OpenSSL.
+ *
+ * Throws an exception if the version is unknown or not supported.
+ *
+ * @param version String of a TLS version, for example "TLSv1.2".
+ * @return The value of the corresponding TLS*_VERSION macro.
+ */
+int ResolveTlsProtocolVersion(const std::string& version) {
+	if (version == "TLSv1.2") {
+		return TLS1_2_VERSION;
+	} else if (version == "TLSv1.3") {
+#if OPENSSL_VERSION_NUMBER >= 0x10101000L
+		return TLS1_3_VERSION;
+#else /* OPENSSL_VERSION_NUMBER >= 0x10101000L */
+		throw std::runtime_error("'" + version + "' is only supported with OpenSSL 1.1.1 or newer");
+#endif /* OPENSSL_VERSION_NUMBER >= 0x10101000L */
+	} else {
+		throw std::runtime_error("Unknown TLS protocol version '" + version + "'");
+	}
+}
+
+Shared<boost::asio::ssl::context>::Ptr SetupSslContext(String certPath, String keyPath,
+	String caPath, String crlPath, String cipherList, String protocolmin, DebugInfo di)
+{
+	namespace ssl = boost::asio::ssl;
+
+	Shared<ssl::context>::Ptr context;
+
+	try {
+		context = MakeAsioSslContext(certPath, keyPath, caPath);
+	} catch (const std::exception&) {
+		BOOST_THROW_EXCEPTION(ScriptError("Cannot make SSL context for cert path: '"
+			+ certPath + "' key path: '" + keyPath + "' ca path: '" + caPath + "'.", di));
+	}
+
+	if (!crlPath.IsEmpty()) {
+		try {
+			AddCRLToSSLContext(context, crlPath);
+		} catch (const std::exception&) {
+			BOOST_THROW_EXCEPTION(ScriptError("Cannot add certificate revocation list to SSL context for crl path: '"
+				+ crlPath + "'.", di));
+		}
+	}
+
+	if (!cipherList.IsEmpty()) {
+		try {
+			SetCipherListToSSLContext(context, cipherList);
+		} catch (const std::exception&) {
+			BOOST_THROW_EXCEPTION(ScriptError("Cannot set cipher list to SSL context for cipher list: '"
+				+ cipherList + "'.", di));
+		}
+	}
+
+	if (!protocolmin.IsEmpty()){
+		try {
+			SetTlsProtocolminToSSLContext(context, protocolmin);
+		} catch (const std::exception&) {
+			BOOST_THROW_EXCEPTION(ScriptError("Cannot set minimum TLS protocol version to SSL context with tls_protocolmin: '" + protocolmin + "'.", di));
+		}
+	}
+
+	return std::move(context);
+}
+
 /**
  * Set the minimum TLS protocol version to the specified SSL context.
  *
@@ -235,16 +309,24 @@ void SetCipherListToSSLContext(const Shared<boost::asio::ssl::context>::Ptr& con
  */
 void SetTlsProtocolminToSSLContext(const Shared<boost::asio::ssl::context>::Ptr& context, const String& tlsProtocolmin)
 {
-	// tlsProtocolmin has no effect since we enforce TLS 1.2 since 2.11.
-	/*
-	std::shared_ptr<SSL_CTX> sslContext = std::shared_ptr<SSL_CTX>(context->native_handle());
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+	int ret = SSL_CTX_set_min_proto_version(context->native_handle(), ResolveTlsProtocolVersion(tlsProtocolmin));
 
-	long flags = SSL_CTX_get_options(sslContext.get());
+	if (ret != 1) {
+		char errbuf[256];
 
-	flags |= ...;
-
-	SSL_CTX_set_options(sslContext.get(), flags);
-	*/
+		ERR_error_string_n(ERR_peek_error(), errbuf, sizeof errbuf);
+		Log(LogCritical, "SSL")
+			<< "Error setting minimum TLS protocol version: " << ERR_peek_error() << ", \"" << errbuf << "\"";
+		BOOST_THROW_EXCEPTION(openssl_error()
+			<< boost::errinfo_api_function("SSL_CTX_set_min_proto_version")
+			<< errinfo_openssl_error(ERR_peek_error()));
+	}
+#else /* OPENSSL_VERSION_NUMBER >= 0x10100000L */
+	// This should never happen. On this OpenSSL version, ResolveTlsProtocolVersion() should either return TLS 1.2
+	// or throw an exception, as that's the only TLS version supported by both Icinga and ancient OpenSSL.
+	VERIFY(ResolveTlsProtocolVersion(tlsProtocolmin) == TLS1_2_VERSION);
+#endif /* OPENSSL_VERSION_NUMBER >= 0x10100000L */
 }
 
 /**
@@ -541,7 +623,7 @@ std::shared_ptr<X509> CreateCert(EVP_PKEY *pubkey, X509_NAME *subject, X509_NAME
 	X509 *cert = X509_new();
 	X509_set_version(cert, 2);
 	X509_gmtime_adj(X509_get_notBefore(cert), 0);
-	X509_gmtime_adj(X509_get_notAfter(cert), ca ? ROOT_VALID_FOR : LEAF_VALID_FOR);
+	X509_gmtime_adj(X509_get_notAfter(cert), 365 * 24 * 60 * 60 * 15);
 	X509_set_pubkey(cert, pubkey);
 
 	X509_set_subject_name(cert, subject);
@@ -670,24 +752,10 @@ std::shared_ptr<X509> CreateCertIcingaCA(const std::shared_ptr<X509>& cert)
 	return CreateCertIcingaCA(pkey.get(), X509_get_subject_name(cert.get()));
 }
 
-bool IsCertUptodate(const std::shared_ptr<X509>& cert)
-{
-	time_t now;
-	time(&now);
-
-	/* auto-renew all certificates which were created before 2017 to force an update of the CA,
-	 * because Icinga versions older than 2.4 sometimes create certificates with an invalid
-	 * serial number. */
-	time_t forceRenewalEnd = 1483228800; /* January 1st, 2017 */
-	time_t renewalStart = now + RENEW_THRESHOLD;
-
-	return X509_cmp_time(X509_get_notBefore(cert.get()), &forceRenewalEnd) != -1 && X509_cmp_time(X509_get_notAfter(cert.get()), &renewalStart) != -1;
-}
-
-String CertificateToString(X509* cert)
+String CertificateToString(const std::shared_ptr<X509>& cert)
 {
 	BIO *mem = BIO_new(BIO_s_mem());
-	PEM_write_bio_X509(mem, cert);
+	PEM_write_bio_X509(mem, cert.get());
 
 	char *data;
 	long len = BIO_get_mem_data(mem, &data);
@@ -776,9 +844,13 @@ String SHA1(const String& s, bool binary)
 	if (binary)
 		return String(reinterpret_cast<const char*>(digest), reinterpret_cast<const char *>(digest + SHA_DIGEST_LENGTH));
 
+	static const char hexdigits[] = "0123456789abcdef";
 	char output[SHA_DIGEST_LENGTH*2+1];
-	for (int i = 0; i < 20; i++)
-		sprintf(output + 2 * i, "%02x", digest[i]);
+	for (int i = 0; i < SHA_DIGEST_LENGTH; i++) {
+		output[2*i] = hexdigits[digest[i] >> 4];
+		output[2*i + 1] = hexdigits[digest[i] & 0xf];
+	}
+	output[2*SHA_DIGEST_LENGTH] = 0;
 
 	return output;
 }
@@ -830,7 +902,7 @@ String RandomString(int length)
 	/* Ensure that password generation is atomic. RAND_bytes is not thread-safe
 	 * in OpenSSL < 1.1.0.
 	 */
-	boost::mutex::scoped_lock lock(l_RandomMutex);
+	std::unique_lock<std::mutex> lock(l_RandomMutex);
 
 	if (!RAND_bytes(bytes, length)) {
 		delete [] bytes;
@@ -860,47 +932,27 @@ String RandomString(int length)
 
 bool VerifyCertificate(const std::shared_ptr<X509> &caCertificate, const std::shared_ptr<X509> &certificate, const String& crlFile)
 {
-	return VerifyCertificate(caCertificate.get(), certificate.get(), crlFile);
-}
-
-bool VerifyCertificate(X509* caCertificate, X509* certificate, const String& crlFile)
-{
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-	/*
-	 * OpenSSL older than version 1.1.0 stored a valid flag in the struct behind X509* which leads to certain validation
-	 * steps to be skipped on subsequent verification operations. If a certificate is verified multiple times with a
-	 * different configuration, for example with different trust anchors, this can result in the certificate
-	 * incorrectly being treated as valid.
-	 *
-	 * This issue is worked around by serializing and deserializing the certificate which creates a new struct instance
-	 * with the valid flag cleared, hence performing the full validation.
-	 *
-	 * The flag in question was removed in OpenSSL 1.1.0, so this extra step isn't necessary for more recent versions:
-	 * https://github.com/openssl/openssl/commit/0e76014e584ba78ef1d6ecb4572391ef61c4fb51
-	 */
-	std::shared_ptr<X509> copy = StringToCertificate(CertificateToString(certificate));
-	VERIFY(copy.get() != certificate);
-	certificate = copy.get();
-#endif
-
-	std::unique_ptr<X509_STORE, decltype(&X509_STORE_free)> store{X509_STORE_new(), &X509_STORE_free};
+	X509_STORE *store = X509_STORE_new();
 
 	if (!store)
 		return false;
 
-	X509_STORE_add_cert(store.get(), caCertificate);
+	X509_STORE_add_cert(store, caCertificate.get());
 
 	if (!crlFile.IsEmpty()) {
-		AddCRLToSSLContext(store.get(), crlFile);
+		AddCRLToSSLContext(store, crlFile);
 	}
 
-	std::unique_ptr<X509_STORE_CTX, decltype(&X509_STORE_CTX_free)> csc{X509_STORE_CTX_new(), &X509_STORE_CTX_free};
-	X509_STORE_CTX_init(csc.get(), store.get(), certificate, nullptr);
+	X509_STORE_CTX *csc = X509_STORE_CTX_new();
+	X509_STORE_CTX_init(csc, store, certificate.get(), nullptr);
 
-	int rc = X509_verify_cert(csc.get());
+	int rc = X509_verify_cert(csc);
+
+	X509_STORE_CTX_free(csc);
+	X509_STORE_free(store);
 
 	if (rc == 0) {
-		int err = X509_STORE_CTX_get_error(csc.get());
+		int err = X509_STORE_CTX_get_error(csc);
 
 		BOOST_THROW_EXCEPTION(openssl_error()
 			<< boost::errinfo_api_function("X509_verify_cert")
