@@ -23,6 +23,16 @@ REGISTER_TYPE(IdoPgsqlConnection);
 
 REGISTER_STATSFUNCTION(IdoPgsqlConnection, &IdoPgsqlConnection::StatsFunc);
 
+const char * IdoPgsqlConnection::GetLatestSchemaVersion() const noexcept
+{
+	return "1.14.3";
+}
+
+const char * IdoPgsqlConnection::GetCompatSchemaVersion() const noexcept
+{
+	return "1.14.3";
+}
+
 IdoPgsqlConnection::IdoPgsqlConnection()
 {
 	m_QueryQueue.SetName("IdoPgsqlConnection, " + GetName());
@@ -77,19 +87,19 @@ void IdoPgsqlConnection::Resume()
 
 	SetConnected(false);
 
-	m_QueryQueue.SetExceptionCallback(std::bind(&IdoPgsqlConnection::ExceptionHandler, this, _1));
+	m_QueryQueue.SetExceptionCallback([this](boost::exception_ptr exp) { ExceptionHandler(std::move(exp)); });
 
 	/* Immediately try to connect on Resume() without timer. */
-	m_QueryQueue.Enqueue(std::bind(&IdoPgsqlConnection::Reconnect, this), PriorityImmediate);
+	m_QueryQueue.Enqueue([this]() { Reconnect(); }, PriorityImmediate);
 
 	m_TxTimer = new Timer();
 	m_TxTimer->SetInterval(1);
-	m_TxTimer->OnTimerExpired.connect(std::bind(&IdoPgsqlConnection::TxTimerHandler, this));
+	m_TxTimer->OnTimerExpired.connect([this](const Timer * const&) { NewTransaction(); });
 	m_TxTimer->Start();
 
 	m_ReconnectTimer = new Timer();
 	m_ReconnectTimer->SetInterval(10);
-	m_ReconnectTimer->OnTimerExpired.connect(std::bind(&IdoPgsqlConnection::ReconnectTimerHandler, this));
+	m_ReconnectTimer->OnTimerExpired.connect([this](const Timer * const&) { ReconnectTimerHandler(); });
 	m_ReconnectTimer->Start();
 
 	/* Start with queries after connect. */
@@ -104,11 +114,6 @@ void IdoPgsqlConnection::Pause()
 
 	m_ReconnectTimer.reset();
 
-	m_QueryQueue.Enqueue(std::bind(&IdoPgsqlConnection::Disconnect, this), PriorityLow);
-
-	/* Work on remaining tasks but never delete the threads, for HA resuming later. */
-	m_QueryQueue.Join();
-
 	Log(LogInformation, "IdoPgsqlConnection")
 		<< "'" << GetName() << "' paused.";
 }
@@ -148,17 +153,12 @@ void IdoPgsqlConnection::Disconnect()
 		<< "Disconnected from '" << GetName() << "' database '" << GetDatabase() << "'.";
 }
 
-void IdoPgsqlConnection::TxTimerHandler()
-{
-	NewTransaction();
-}
-
 void IdoPgsqlConnection::NewTransaction()
 {
 	if (IsPaused())
 		return;
 
-	m_QueryQueue.Enqueue(std::bind(&IdoPgsqlConnection::InternalNewTransaction, this), PriorityNormal, true);
+	m_QueryQueue.Enqueue([this]() { InternalNewTransaction(); }, PriorityNormal, true);
 }
 
 void IdoPgsqlConnection::InternalNewTransaction()
@@ -176,7 +176,7 @@ void IdoPgsqlConnection::InternalNewTransaction()
 void IdoPgsqlConnection::ReconnectTimerHandler()
 {
 	/* Only allow Reconnect events with high priority. */
-	m_QueryQueue.Enqueue(std::bind(&IdoPgsqlConnection::Reconnect, this), PriorityHigh);
+	m_QueryQueue.Enqueue([this]() { Reconnect(); }, PriorityHigh);
 }
 
 void IdoPgsqlConnection::Reconnect()
@@ -261,14 +261,6 @@ void IdoPgsqlConnection::Reconnect()
 
 	IdoPgsqlResult result;
 
-	/* explicitely require legacy mode for string escaping in PostgreSQL >= 9.1
-	 * changing standard_conforming_strings to on by default
-	 */
-	if (m_Pgsql->serverVersion(m_Connection) >= 90100) {
-		IncreasePendingQueries(1);
-		result = Query("SET standard_conforming_strings TO off");
-	}
-
 	String dbVersionName = "idoutils";
 	IncreasePendingQueries(1);
 	result = Query("SELECT version FROM " + GetTablePrefix() + "dbversion WHERE name=E'" + Escape(dbVersionName) + "'");
@@ -288,13 +280,13 @@ void IdoPgsqlConnection::Reconnect()
 
 	SetSchemaVersion(version);
 
-	if (Utility::CompareVersion(IDO_COMPAT_SCHEMA_VERSION, version) < 0) {
+	if (Utility::CompareVersion(GetCompatSchemaVersion(), version) < 0) {
 		m_Pgsql->finish(m_Connection);
 		SetConnected(false);
 
 		Log(LogCritical, "IdoPgsqlConnection")
 			<< "Schema version '" << version << "' does not match the required version '"
-			<< IDO_COMPAT_SCHEMA_VERSION << "' (or newer)! Please check the upgrade documentation at "
+			<< GetCompatSchemaVersion() << "' (or newer)! Please check the upgrade documentation at "
 			<< "https://icinga.com/docs/icinga2/latest/doc/16-upgrading-icinga-2/#upgrading-postgresql-db";
 
 		BOOST_THROW_EXCEPTION(std::runtime_error("Schema version mismatch."));
@@ -439,9 +431,9 @@ void IdoPgsqlConnection::Reconnect()
 
 	UpdateAllObjects();
 
-	m_QueryQueue.Enqueue(std::bind(&IdoPgsqlConnection::ClearTablesBySession, this), PriorityNormal);
+	m_QueryQueue.Enqueue([this]() { ClearTablesBySession(); }, PriorityNormal);
 
-	m_QueryQueue.Enqueue(std::bind(&IdoPgsqlConnection::FinishConnect, this, startTime), PriorityNormal);
+	m_QueryQueue.Enqueue([this, startTime]() { FinishConnect(startTime); }, PriorityNormal);
 }
 
 void IdoPgsqlConnection::FinishConnect(double startTime)
@@ -522,7 +514,7 @@ IdoPgsqlResult IdoPgsqlConnection::Query(const String& query)
 		);
 	}
 
-	return IdoPgsqlResult(result, std::bind(&PgsqlInterface::clear, std::cref(m_Pgsql), _1));
+	return IdoPgsqlResult(result, [this](PGresult* result) { m_Pgsql->clear(result); });
 }
 
 DbReference IdoPgsqlConnection::GetSequenceValue(const String& table, const String& column)
@@ -595,7 +587,7 @@ void IdoPgsqlConnection::ActivateObject(const DbObject::Ptr& dbobj)
 	if (IsPaused())
 		return;
 
-	m_QueryQueue.Enqueue(std::bind(&IdoPgsqlConnection::InternalActivateObject, this, dbobj), PriorityNormal);
+	m_QueryQueue.Enqueue([this, dbobj]() { InternalActivateObject(dbobj); }, PriorityNormal);
 }
 
 void IdoPgsqlConnection::InternalActivateObject(const DbObject::Ptr& dbobj)
@@ -634,7 +626,7 @@ void IdoPgsqlConnection::DeactivateObject(const DbObject::Ptr& dbobj)
 	if (IsPaused())
 		return;
 
-	m_QueryQueue.Enqueue(std::bind(&IdoPgsqlConnection::InternalDeactivateObject, this, dbobj), PriorityNormal);
+	m_QueryQueue.Enqueue([this, dbobj]() { InternalDeactivateObject(dbobj); }, PriorityNormal);
 }
 
 void IdoPgsqlConnection::InternalDeactivateObject(const DbObject::Ptr& dbobj)
@@ -656,6 +648,8 @@ void IdoPgsqlConnection::InternalDeactivateObject(const DbObject::Ptr& dbobj)
 
 	/* Note that we're _NOT_ clearing the db refs via SetReference/SetConfigUpdate/SetStatusUpdate
 	 * because the object is still in the database. */
+
+	SetObjectActive(dbobj, false);
 }
 
 bool IdoPgsqlConnection::FieldToEscapedString(const String& key, const Value& value, Value *result)
@@ -670,7 +664,9 @@ bool IdoPgsqlConnection::FieldToEscapedString(const String& key, const Value& va
 
 	Value rawvalue = DbValue::ExtractValue(value);
 
-	if (rawvalue.IsObjectType<ConfigObject>()) {
+	if (rawvalue.GetType() == ValueEmpty) {
+		*result = "NULL";
+	} else if (rawvalue.IsObjectType<ConfigObject>()) {
 		DbObject::Ptr dbobjcol = DbObject::GetOrCreateByObject(rawvalue);
 
 		if (!dbobjcol) {
@@ -731,13 +727,13 @@ bool IdoPgsqlConnection::FieldToEscapedString(const String& key, const Value& va
 
 void IdoPgsqlConnection::ExecuteQuery(const DbQuery& query)
 {
-	if (IsPaused())
+	if (IsPaused() && GetPauseCalled())
 		return;
 
 	ASSERT(query.Category != DbCatInvalid);
 
 	IncreasePendingQueries(1);
-	m_QueryQueue.Enqueue(std::bind(&IdoPgsqlConnection::InternalExecuteQuery, this, query, -1), query.Priority, true);
+	m_QueryQueue.Enqueue([this, query]() { InternalExecuteQuery(query, -1); }, query.Priority, true);
 }
 
 void IdoPgsqlConnection::ExecuteMultipleQueries(const std::vector<DbQuery>& queries)
@@ -749,7 +745,7 @@ void IdoPgsqlConnection::ExecuteMultipleQueries(const std::vector<DbQuery>& quer
 		return;
 
 	IncreasePendingQueries(queries.size());
-	m_QueryQueue.Enqueue(std::bind(&IdoPgsqlConnection::InternalExecuteMultipleQueries, this, queries), queries[0].Priority, true);
+	m_QueryQueue.Enqueue([this, queries]() { InternalExecuteMultipleQueries(queries); }, queries[0].Priority, true);
 }
 
 bool IdoPgsqlConnection::CanExecuteQuery(const DbQuery& query)
@@ -773,9 +769,6 @@ bool IdoPgsqlConnection::CanExecuteQuery(const DbQuery& query)
 		for (const Dictionary::Pair& kv : query.Fields) {
 			Value value;
 
-			if (kv.second.IsEmpty() && !kv.second.IsString())
-				continue;
-
 			if (!FieldToEscapedString(kv.first, kv.second, &value))
 				return false;
 		}
@@ -802,7 +795,7 @@ void IdoPgsqlConnection::InternalExecuteMultipleQueries(const std::vector<DbQuer
 		ASSERT(query.Type == DbQueryNewTransaction || query.Category != DbCatInvalid);
 
 		if (!CanExecuteQuery(query)) {
-			m_QueryQueue.Enqueue(std::bind(&IdoPgsqlConnection::InternalExecuteMultipleQueries, this, queries), query.Priority);
+			m_QueryQueue.Enqueue([this, queries]() { InternalExecuteMultipleQueries(queries); }, query.Priority);
 			return;
 		}
 	}
@@ -816,7 +809,7 @@ void IdoPgsqlConnection::InternalExecuteQuery(const DbQuery& query, int typeOver
 {
 	AssertOnWorkQueue();
 
-	if (IsPaused()) {
+	if (IsPaused() && GetPauseCalled()) {
 		DecreasePendingQueries(1);
 		return;
 	}
@@ -845,7 +838,7 @@ void IdoPgsqlConnection::InternalExecuteQuery(const DbQuery& query, int typeOver
 
 	/* check if there are missing object/insert ids and re-enqueue the query */
 	if (!CanExecuteQuery(query)) {
-		m_QueryQueue.Enqueue(std::bind(&IdoPgsqlConnection::InternalExecuteQuery, this, query, typeOverride), query.Priority);
+		m_QueryQueue.Enqueue([this, query, typeOverride]() { InternalExecuteQuery(query, typeOverride); }, query.Priority);
 		return;
 	}
 
@@ -861,7 +854,7 @@ void IdoPgsqlConnection::InternalExecuteQuery(const DbQuery& query, int typeOver
 
 		for (const Dictionary::Pair& kv : query.WhereCriteria) {
 			if (!FieldToEscapedString(kv.first, kv.second, &value)) {
-				m_QueryQueue.Enqueue(std::bind(&IdoPgsqlConnection::InternalExecuteQuery, this, query, -1), query.Priority);
+				m_QueryQueue.Enqueue([this, query]() { InternalExecuteQuery(query, -1); }, query.Priority);
 				return;
 			}
 
@@ -929,11 +922,8 @@ void IdoPgsqlConnection::InternalExecuteQuery(const DbQuery& query, int typeOver
 		Value value;
 		bool first = true;
 		for (const Dictionary::Pair& kv : query.Fields) {
-			if (kv.second.IsEmpty() && !kv.second.IsString())
-				continue;
-
 			if (!FieldToEscapedString(kv.first, kv.second, &value)) {
-				m_QueryQueue.Enqueue(std::bind(&IdoPgsqlConnection::InternalExecuteQuery, this, query, -1), query.Priority);
+				m_QueryQueue.Enqueue([this, query]() { InternalExecuteQuery(query, -1); }, query.Priority);
 				return;
 			}
 
@@ -998,7 +988,7 @@ void IdoPgsqlConnection::CleanUpExecuteQuery(const String& table, const String&
 		return;
 
 	IncreasePendingQueries(1);
-	m_QueryQueue.Enqueue(std::bind(&IdoPgsqlConnection::InternalCleanUpExecuteQuery, this, table, time_column, max_age), PriorityLow, true);
+	m_QueryQueue.Enqueue([this, table, time_column, max_age]() { InternalCleanUpExecuteQuery(table, time_column, max_age); }, PriorityLow, true);
 }
 
 void IdoPgsqlConnection::InternalCleanUpExecuteQuery(const String& table, const String& time_column, double max_age)
